@@ -44,6 +44,8 @@
 #include "apr.h"
 #include "apr_strings.h"
 #include "apr_lib.h"
+#include "apr_version.h"
+#include "apu_version.h"
 #define APR_WANT_STRFUNC
 #include "apr_want.h"
 
@@ -351,6 +353,18 @@ static int show_server_settings(request_rec * r)
                "<dt><strong>Server Built:</strong> "
                "<font size=\"+1\"><tt>%s</tt></font></dt>\n",
                ap_get_server_built());
+    ap_rprintf(r,
+               "<dt><strong>Server loaded APR Version:</strong> "
+               "<tt>%s</tt></dt>\n", apr_version_string());
+    ap_rprintf(r,
+               "<dt><strong>Compiled with APR Version:</strong> "
+               "<tt>%s</tt></dt>\n", APR_VERSION_STRING);
+    ap_rprintf(r,
+               "<dt><strong>Server loaded APU Version:</strong> "
+               "<tt>%s</tt></dt>\n", apu_version_string());
+    ap_rprintf(r,
+               "<dt><strong>Compiled with APU Version:</strong> "
+               "<tt>%s</tt></dt>\n", APU_VERSION_STRING);
     ap_rprintf(r,
                "<dt><strong>Module Magic Number:</strong> "
                "<tt>%d:%d</tt></dt>\n", MODULE_MAGIC_NUMBER_MAJOR,
@@ -364,7 +378,7 @@ static int show_server_settings(request_rec * r)
                "<tt>connection: %d &nbsp;&nbsp; "
                "keep-alive: %d</tt></dt>",
                (int) (apr_time_sec(serv->timeout)),
-               (int) (apr_time_sec(serv->timeout)));
+               (int) (apr_time_sec(serv->keep_alive_timeout)));
     ap_mpm_query(AP_MPMQ_MAX_DAEMON_USED, &max_daemons);
     ap_mpm_query(AP_MPMQ_IS_THREADED, &threaded);
     ap_mpm_query(AP_MPMQ_IS_FORKED, &forked);
@@ -607,7 +621,7 @@ static int display_info(request_rec * r)
     if (r->method_number != M_GET)
         return DECLINED;
 
-    ap_set_content_type(r, "text/html");
+    ap_set_content_type(r, "text/html; charset=ISO-8859-1");
 
     ap_rputs(DOCTYPE_XHTML_1_0T
              "<html xmlns=\"http://www.w3.org/1999/xhtml\">\n"
@@ -721,7 +735,7 @@ static int display_info(request_rec * r)
                                            ap_escape_html(r->pool, cmd->name),
                                            cmd->name[0] == '<' ? "&gt;" : "");
                                 if (cmd->errmsg) {
-                                    ap_rputs(cmd->errmsg, r);
+                                    ap_rputs(ap_escape_html(r->pool, cmd->errmsg), r);
                                 }
                                 ap_rputs("</i></tt></dd>\n", r);
                             }
