@@ -18,6 +18,7 @@
 #include "apr_strings.h"
 #include "apr_arch_proc_mutex.h"
 #include "apr_arch_file_io.h" /* for apr_mkstemp() */
+#include "apr_hash.h"
 
 APR_DECLARE(apr_status_t) apr_proc_mutex_destroy(apr_proc_mutex_t *mutex)
 {
@@ -51,14 +52,27 @@ static apr_status_t proc_mutex_posix_cleanup(void *mutex_)
     return APR_SUCCESS;
 }    
 
+static unsigned int rshash (char *p) {
+    /* hash function from Robert Sedgwicks 'Algorithms in C' book */
+   unsigned int b    = 378551;
+   unsigned int a    = 63689;
+   unsigned int retval = 0;
+
+   for( ; *p; p++)
+   {
+      retval = retval * a + (*p);
+      a *= b;
+   }
+
+   return retval;
+}
+
 static apr_status_t proc_mutex_posix_create(apr_proc_mutex_t *new_mutex,
                                             const char *fname)
 {
+    #define APR_POSIXSEM_NAME_MIN 13
     sem_t *psem;
-    char semname[31];
-    apr_time_t now;
-    unsigned long sec;
-    unsigned long usec;
+    char semname[32];
     
     new_mutex->interproc = apr_palloc(new_mutex->pool,
                                       sizeof(*new_mutex->interproc));
@@ -69,28 +83,42 @@ static apr_status_t proc_mutex_posix_create(apr_proc_mutex_t *new_mutex,
      *   - be at most 14 chars
      *   - be unique and not match anything on the filesystem
      *
-     * Because of this, we ignore fname, and try our
-     * own naming system. We tuck the name away, since it might
-     * be useful for debugging. to  make this as robust as possible,
-     * we initially try something larger (and hopefully more unique)
-     * and gracefully fail down to the LCD above.
+     * Because of this, we use fname to generate a (unique) hash
+     * and use that as the name of the semaphore. If no filename was
+     * given, we create one based on the time. We tuck the name
+     * away, since it might be useful for debugging. We use 2 hashing
+     * functions to try to avoid collisions.
+     *
+     * To  make this as robust as possible, we initially try something
+     * larger (and hopefully more unique) and gracefully fail down to the
+     * LCD above.
      *
      * NOTE: Darwin (Mac OS X) seems to be the most restrictive
      * implementation. Versions previous to Darwin 6.2 had the 14
      * char limit, but later rev's allow up to 31 characters.
      *
      */
-    now = apr_time_now();
-    sec = apr_time_sec(now);
-    usec = apr_time_usec(now);
-    apr_snprintf(semname, sizeof(semname), "/ApR.%lxZ%lx", sec, usec);
+    if (fname) {
+        apr_ssize_t flen = strlen(fname);
+        char *p = apr_pstrndup(new_mutex->pool, fname, strlen(fname));
+        unsigned int h1, h2;
+        h1 = apr_hashfunc_default((const char *)p, &flen);
+        h2 = rshash(p);
+        apr_snprintf(semname, sizeof(semname), "/ApR.%xH%x", h1, h2);
+    } else {
+        apr_time_t now;
+        unsigned long sec;
+        unsigned long usec;
+        now = apr_time_now();
+        sec = apr_time_sec(now);
+        usec = apr_time_usec(now);
+        apr_snprintf(semname, sizeof(semname), "/ApR.%lxZ%lx", sec, usec);
+    }
     psem = sem_open(semname, O_CREAT | O_EXCL, 0644, 1);
     if (psem == (sem_t *)SEM_FAILED) {
         if (errno == ENAMETOOLONG) {
             /* Oh well, good try */
-            semname[13] = '\0';
-        } else if (errno == EEXIST) {
-            apr_snprintf(semname, sizeof(semname), "/ApR.%lxZ%lx", usec, sec);
+            semname[APR_POSIXSEM_NAME_MIN] = '\0';
         } else {
             return errno;
         }
